@@ -1,6 +1,4 @@
-//! Transition system parser.
-//!
-//! The [`DEMO` constant][crate::DEMO] illustrates and discusses the syntax expected by the parser.
+//! Backend parser (SMT-LIB) and frontend parser (requires the `parser` feature).
 
 crate::prelude!();
 
@@ -36,6 +34,8 @@ pub fn fail_if_kw(ident: impl AsRef<str>) -> Result<(), String> {
 #[cfg(feature = "parser")]
 peg::parser! {
     /// PEG parsing rules, requires the `parser` feature.
+    ///
+    /// The [`DEMO` constant][crate::DEMO] illustrates and discusses the syntax expected by the parser.
     pub grammar rules() for str {
         /// Whitespace.
         pub rule whitespace() = quiet! {
@@ -476,57 +476,60 @@ peg::parser! {
         }
         / expected!("a type (`int`, `rat` or `bool`")
 
-        /// Parses a state declaration.
+        /// Parses some state variables.
         ///
-        /// A state declaration is a list of `<ident> : <type>`. No separator is needed. For
-        /// convenience, state variables with the same type can be listed together, separated
-        /// by `,`, before the `: <type>`. For instance, `v_1, v_2, v_3: int`.
+        /// A state declaration is a comma-separated list of `<ident> : <type>`, with an optional
+        /// trailing comma. For convenience, state variables with the same type can be listed
+        /// together, separated by whitespace(s), before the `: <type>`. For instance, `v_1 v_2 v_3:
+        /// int`.
         ///
         /// # Examples
         ///
         /// ```rust
-        /// # use mikino_api::parse::rules::state;
+        /// # use mikino_api::parse::rules::svars;
         /// let input = "\
-        ///     n_1 : int
-        ///     b_1 : bool
+        ///     n_1 : int,
+        ///     b_1 : bool,
         ///     // Avoid putting declarations on the same line, it looks bad.
-        ///     n_2: int b_2 : bool
+        ///     n_2: int, b_2 : bool,
         ///     // Aggregated declaration.
-        ///     n_3, n_4, n_5: int
-        ///     p, q: bool\
+        ///     n_3 n_4 n_5: int,
+        ///     p q: bool,\
         /// ";
-        /// let decls = state(input).unwrap().unwrap();
+        /// let decls = svars(input).unwrap().unwrap();
         /// assert_eq!(
         ///     decls.to_string(),
         ///     // State-declaration-printing sorts and aggregates idents alphabetically.
         ///     "\
-        /// b_1, b_2: bool
-        /// n_1, n_2, n_3, n_4, n_5: int
-        /// p, q: bool\
+        /// b_1 b_2: bool,
+        /// n_1 n_2 n_3 n_4 n_5: int,
+        /// p q: bool,\
         ///     "
         /// );
         /// ```
-        pub rule state() -> PRes<trans::Decls>
-        = state:(
+        pub rule svars() -> PRes<trans::Decls>
+        = svars:(
             quiet! {
+                _
                 svar_doc:outer_doc()
                 _
                 svar:ident()
                 svars:(
-                    _ ","
+                    _
                     svar_doc:outer_doc()
                     _ id:ident() {
                         id
                     }
                 )*
-                _ ":" _ svars_typ:hsmt_typ() {
+                _ ":" _ svars_typ:hsmt_typ()
+                {
                     (svar, svars, svars_typ)
                 }
             }
             / expected!(r#"list of "<ident>, <ident>, ... : <type>""#)
-        )+ {
+        ) ++ (_ "," _) (",")? {
             let mut decls = trans::Decls::new();
-            for (svar, svars, typ) in state {
+            for (svar, svars, typ) in svars {
                 for svar in Some(svar).into_iter().chain(svars) {
                     let prev = decls.register(svar.inner, typ);
                     if prev.is_some() {
@@ -551,9 +554,9 @@ peg::parser! {
         /// ```rust
         /// # use mikino_api::parse::rules::candidates;
         /// let input = r#"
-        ///     "some candidate": x ≥ 0
-        ///     "another one": x ≥ y + 2 ⋁ y ≥ -7
-        ///     "tautology": p ⋁ ¬p"#;
+        ///     "some candidate": x ≥ 0,
+        ///     "another one": x ≥ y + 2 ⋁ y ≥ -7,
+        ///     "tautology": p ⋁ ¬p,"#;
         /// let mut candidates = candidates(input).unwrap().into_iter();
         ///
         /// let (name, expr) = candidates.next().unwrap();
@@ -570,14 +573,17 @@ peg::parser! {
         /// ```
         pub rule candidates() -> Vec<(Spn<&'input str>, Ast<'input>)>
         = quiet! {
-            (
+            cands:(
                 _ s:position!() name:(
                     "\"" name:$( ("\\\"" / [^'"'])* ) "\"" { name }
                 ) e:position!() _ ":" _
-                expr:hsmt() {
+                expr:hsmt()
+                {
                     (Spn::new(name, (s, e)), expr)
                 }
-            )+
+            ) ++ (_ "," _) (",")? {
+                cands
+            }
         }
         / expected!(r#"list of "<name> : <expr>" where <name> is a double-quoted string"#)
 
@@ -588,24 +594,28 @@ peg::parser! {
         =
         sys_doc:inner_doc()
 
-        state_doc:outer_doc()
-        _ "state" _ "{" _ decls:state() _ "}"
-        state_doc:outer_doc()
-        _ "init" _ "{" _ init:(
-            quiet! { hsmt() }
-            / expected!("stateless expression")
+        vars_doc:outer_doc()
+        _ "svars" _ "{" _ decls:svars() _ "}"
+        init_doc:outer_doc()
+        _ init_s:position!() "init" init_e:position!() _ "{" _ hsmt_init:(
+            quiet! {
+                init:(hsmt()) ++ (_ "," _) (",")? { init }
+            }
+            / expected!("comma-separated list of stateless expressions")
          ) _  "}"
-        state_doc:outer_doc()
-        _ "trans" _ "{" _ trans:(
-            quiet! { hsmt() }
-            / expected!("stateful expression")
+        trans_doc:outer_doc()
+        _ trans_s:position!() "trans" trans_e:position!() _ "{" _ hsmt_trans:(
+            quiet! {
+                trans:(hsmt()) ++ (_ "," _) (",")? { trans }
+            }
+            / expected!("comma-separated list of stateful expressions")
          ) _ "}"
-        state_doc:outer_doc()
+        candidates_doc:outer_doc()
         _ "candidates" _ "{" _ candidates:candidates() _ "}"
         _ {
             let decls = decls?;
-            let init = init.to_expr(&decls)?;
-            let trans = trans.to_sexpr(&decls)?;
+            let init = Ast::app(Spn::new(Op::And, (init_s, init_e)), hsmt_init).to_expr(&decls)?;
+            let trans = Ast::app(Spn::new(Op::And, (trans_s, trans_e)), hsmt_trans).to_sexpr(&decls)?;
 
             let mut pos = Map::new();
 
@@ -628,7 +638,7 @@ peg::parser! {
 ///
 /// A system is composed of four `{ ... }` blocks each starting with a specific keyword:
 ///
-/// - `state { ... }`: the [state variables][rules::state] of the system;
+/// - `svars { ... }`: the [state variables][rules::svars] of the system;
 ///
 /// - `init { ... }`: the initial predicate, *i.e.* a stateless (no `'` prime) expression;
 ///
